import importlib
import inspect

from cloud_governance.common.logger.init_logger import logger
from cloud_governance.main.environment_variables import environment_variables


class AzurePolicyRunner:
    """
    This method run the azure policies
    """

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self._policy = self.__environment_variables_dict.get('policy')
        self._cloud_name = self.__environment_variables_dict.get('PUBLIC_CLOUD_NAME')

    def run(self):
        """
        Run the azure policies
        @return:
        """
        azure_policies = importlib.import_module(f'cloud_governance.policy.azure.{self._policy}')
        logger.info(f'Account: {self._cloud_name}, Policy: {self._policy}')
        for cls in inspect.getmembers(azure_policies, inspect.isclass):
            if self._policy.replace('_', '') == cls[0].lower():
                response = cls[1]().run()
                if isinstance(response, list) and len(response) > 0:
                    logger.info(f'key: {cls[0]}, count: {len(response)}, {response}')



from typing import Callable

from cloud_governance.common.logger.init_logger import logger
from cloud_governance.policy.policy_runners.common.abstract_policy_runner import AbstractPolicyRunner


class PolicyRunner(AbstractPolicyRunner):

    def execute_policy(self, policy_class_name: str, run_policy: Callable, upload: bool = False):
        """
         This method executes the policy
        :param policy_class_name:
        :type policy_class_name:
        :param run_policy:
        :type run_policy:
        :param upload:
        :type upload:
        :return:
        :rtype:
        """
        policy_result = []
        response = run_policy().run()
        if isinstance(response, str):
            logger.info(response)
        else:
            logger.info(response)
            policy_result.extend(response)
            self._upload_elastic_search.upload(data=policy_result)
        return policy_result

    def __init__(self):
        super().__init__()



from datetime import datetime

import boto3

from cloud_governance.common.clouds.aws.cloudtrail.cloudtrail_operations import CloudTrailOperations
from cloud_governance.common.clouds.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.common.clouds.aws.iam.iam_operations import IAMOperations
from cloud_governance.common.clouds.aws.utils.utils import Utils


class TagClusterOperations:
    """
    This class tags AWS resources
    """

    CLUSTER_ID_COST_ALLOCATION_TAG = 'cluster_id'

    def __init__(self, region: str, input_tags: dict = None,  cluster_name: str = None,  cluster_prefix: str = None, dry_run: str = None, cluster_only: bool = None):
        self.cluster_only = cluster_only
        self.cluster_prefix = cluster_prefix
        self.utils = Utils(region=region)
        self.ec2_client = boto3.client('ec2', region_name=region)
        self.elb_client = boto3.client('elb', region_name=region)
        self.elbv2_client = boto3.client('elbv2', region_name=region)
        self.iam_client = boto3.client('iam', region_name=region)
        self.s3_client = boto3.client('s3')
        self.ec2_operations = EC2Operations(region=region)
        self.iam_operations = IAMOperations()
        self.cluster_name = cluster_name
        self.input_tags = input_tags
        self.cloudtrail = CloudTrailOperations(region_name='us-east-1')
        self._get_username_from_instance_id_and_time = CloudTrailOperations(region_name=region).get_username_by_instance_id_and_time
        self.dry_run = dry_run
        self.iam_users = self.iam_operations.get_iam_users_list()

    def _input_tags_list_builder(self):
        """
        This method build tags list according to input tags dictionary
        @return:
        """
        tags_list = []
        for key, value in self.input_tags.items():
            tags_list.append({'Key': key, 'Value': value})
        return tags_list

    def _get_instances_data(self):
        """
        This method go over all instances
        @return:
        """
        instances_list = []
        ec2s_data = self.ec2_operations.get_instances()
        for items in ec2s_data:
            if items.get('Instances'):
                instances_list.append(items['Instances'])
        return instances_list

    def _fill_na_tags(self, user: str = None):
        """
        This method fill the NA tags
        @param user:
        @return:
        """
        keys = ['User', 'Owner', 'Manager', 'Project', 'Environment', 'Email']
        tags = []
        value = 'NA'
        for key in keys:
            if user and user == 'User':
                value = user
            elif user and user == 'Email':
                value = f'{user}@redhat.com'
            else:
                tags.append({'Key': key, 'Value': value})
        return tags

    def get_user_name_from_name_tag(self, tags: list):
        """
        This method retuns the username from the name tag verified  with iam users
        :param tags:
        :return:
        """
        user_name = self.ec2_operations.get_tag_value_from_tags(tags=tags, tag_name='User')
        if user_name in self.iam_users:
            return user_name
        else:
            name_tag = self.ec2_operations.get_tag_value_from_tags(tags=tags, tag_name='Name')
            for user in self.iam_users:
                if user in name_tag:
                    return user
            return None

    def get_username(self, start_time: datetime, resource_id: str, resource_type: str, tags: list):
        """
        This method returns the username
        :return:
        """
        iam_username = self.get_user_name_from_name_tag(tags=tags)
        if not iam_username:
            return self._get_username_from_instance_id_and_time(start_time=start_time, resource_id=resource_id, resource_type=resource_type)
        return iam_username



from datetime import datetime, timedelta, timezone

import pandas
from cloud_governance.common.elasticsearch.elasticsearch_operations import ElasticSearchOperations
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.common.mails.mail_message import MailMessage
from cloud_governance.common.mails.postfix import Postfix
from cloud_governance.common.utils.utils import Utils
from cloud_governance.main.environment_variables import environment_variables


class SendAggregatedAlerts:
    """
    This class send alerts to users which conditions are not satisfied by the policies
    """

    def __init__(self):
        self.__environment_variables = environment_variables.environment_variables_dict
        self.__days_to_delete_resource = int(self.__environment_variables.get('DAYS_TO_DELETE_RESOURCE'))
        self.__mail_to = self.__environment_variables.get('EMAIL_TO')  # testing purposes
        self.__mail_cc = self.__environment_variables.get('EMAIL_CC', [])
        self.__alert_dry_run = self.__environment_variables.get('ALERT_DRY_RUN')
        self.__mail_message = MailMessage()
        self.__postfix = Postfix()
        self.__es_operations = ElasticSearchOperations()

    def __get_es_data(self):
        """
        This method returns the current day policy data from the elastic_search database
        :return:
        :rtype:
        """
        current_date = (datetime.now(timezone.utc).date()).__str__()
        policy_es_index = self.__environment_variables.get('es_index')
        account_name = (self.__environment_variables.get('account', '').upper()
                        .replace('OPENSHIFT-', '')
                        .replace('OPENSHIFT', '').strip())
        query = {
            "size": 10000,
            "query": {
                "bool": {
                    "must": [
                        {
                            "term": {
                                "account.keyword": {
                                    "value": account_name
                                }
                            }
                        }
                    ],
                    "must_not": [
                        {
                            "terms": {
                                "policy.keyword": [
                                    "ebs_in_use",
                                    "instance_run", "cluster_run", "optimize_resource_report",
                                    "optimize_resources_report", "skipped_resources"
                                ]
                            }
                        }
                    ],
                    "filter": [
                        {
                            "range": {
                                "timestamp": {
                                    "format": "yyyy-MM-dd",
                                    "lte": current_date,
                                    "gte": current_date
                                }
                            }
                        }
                    ]
                }
            }
        }
        records = self.__es_operations.post_query(query=query, es_index=policy_es_index)
        return [record.get('_source') for record in records]

    def __remove_duplicates(self, policy_es_data: list):
        """
        This method removes the duplicate  data
        :return:
        :rtype:
        """
        if policy_es_data:
            df = pandas.DataFrame(policy_es_data)
            df.sort_values(inplace=True, by=['policy'])
            df.fillna(value='', inplace=True)
            df.drop_duplicates(subset='ResourceId', inplace=True)
            return df.to_dict(orient="records")
        return policy_es_data

    def __group_by_policy(self, policy_data: list):
        """
        This method returns the data grouped by policy
        :param policy_data:
        :type policy_data:
        :return:
        :rtype:
        """
        policy_group_data = {}
        for record in policy_data:
            policy_group_data.setdefault(record.get('policy', 'NA'), []).append(record)
        policy_data_list = []
        for _, values in policy_group_data.items():
            policy_data_list.extend(values)
        return policy_data_list

    def __group_by_user(self, policy_data: list):
        """
        This method returns the data grouped by user files
        :param policy_data:
        :type policy_data:
        :return:
        :rtype:
        """
        user_data = {}
        for record in policy_data:
            user_data.setdefault(record.get('User', 'NA'), []).append(record)
        return user_data

    def __update_delete_days(self, policy_es_data: list):
        """
        This method returns the resource delete date
        :param policy_es_data:
        :type policy_es_data:
        :return:
        :rtype:
        """
        filtered_policy_es_data = []
        for record in policy_es_data:
            try:
                days = record.get('ClusterResourcesCount')
                if not days:
                    days = record.get('CleanUpDays')
                if not days:
                    days = record.get('Days')
                if not days:
                    days = record.get('StoppedDays')
                if days:
                    days = int(days)
                if not days:
                    days = 0
                alert_user = True if self.__alert_dry_run else False
                dry_run = record.get('DryRun')
                if record.get('ExpireDays'):
                    days_to_take_action = int(record.get('ExpireDays'))
                else:
                    days_to_take_action = int(self.__days_to_delete_resource)
                if not record.get('SkipPolicy'):
                    record['SkipPolicy'] = 'NA'
                delete_date = ''
                if record.get('SkipPolicy') != 'NA':
                    delete_date = 'skip_delete'
                if days_to_take_action - 5 == days:
                    delete_date = (datetime.utcnow() + timedelta(days=5)).date()
                    alert_user = True
                elif days == days_to_take_action - 3:
                    delete_date = (datetime.utcnow() + timedelta(days=3)).date()
                    alert_user = True
                else:
                    if days >= days_to_take_action:
                        delete_date = datetime.utcnow().date().__str__()
                        alert_user = True
                if record.get('policy') in ['empty_roles', 's3_inactive']:
                    record['RegionName'] = 'us-east-1'
                if Utils.equal_ignore_case(dry_run, 'yes'):
                    record['DeleteDate'] = 'dry_run=yes'
                    filtered_policy_es_data.append(record)
                else:
                    if alert_user:
                        if delete_date != '' and delete_date != 'skip_delete' and Utils.equal_ignore_case(dry_run, 'no'):
                            record['DeleteDate'] = delete_date.__str__()
                            filtered_policy_es_data.append(record)

            except Exception as err:
                raise err
        return filtered_policy_es_data

    def __send_aggregate_email_by_es_data(self):
        """
        This method sends an alert using the elasticsearch data
        :return:
        :rtype:
        """
        policy_es_data = self.__get_es_data()
        policy_es_data = self.__remove_duplicates(policy_es_data=policy_es_data)
        policy_es_data = self.__update_delete_days(policy_es_data)
        if self.__environment_variables.get('ADMIN_MAIL_LIST', ''):
            to_mail_list = self.__environment_variables.get('ADMIN_MAIL_LIST', '')
            group_by_policy = self.__group_by_policy(policy_data=policy_es_data)
            if group_by_policy:
                subject, body = self.__mail_message.get_policy_alert_message(policy_data=group_by_policy)
                self.__postfix.send_email_postfix(subject=subject, content=body, to=to_mail_list, cc=[], mime_type='html')
        else:
            user_policy_data = self.__group_by_user(policy_data=policy_es_data)
            for user, user_records in user_policy_data.items():
                if user_records:
                    subject, body = self.__mail_message.get_policy_alert_message(policy_data=user_records, user=user)
                    self.__postfix.send_email_postfix(subject=subject, content=body, to=user, cc=[],
                                                      mime_type='html')

    @logger_time_stamp
    def run(self):
        """
        This method start the other methods
        :return:
        """
        self.__send_aggregate_email_by_es_data()



import json
from datetime import datetime, timedelta

from cloud_governance.common.clouds.azure.compute.compute_operations import ComputeOperations
from cloud_governance.common.clouds.azure.compute.network_operations import NetworkOperations
from cloud_governance.common.clouds.azure.compute.resource_group_operations import ResourceGroupOperations
from cloud_governance.common.clouds.azure.monitor.monitor_management_operations import MonitorManagementOperations
from cloud_governance.common.utils.configs import INSTANCE_IDLE_DAYS, DEFAULT_ROUND_DIGITS, TOTAL_BYTES_IN_KIB
from cloud_governance.policy.helpers.abstract_policy_operations import AbstractPolicyOperations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.utils.utils import Utils


class AzurePolicyOperations(AbstractPolicyOperations):

    def __init__(self):
        self._cloud_name = 'Azure'
        self.compute_operations = ComputeOperations()
        self.network_operations = NetworkOperations()
        self.resource_group_operations = ResourceGroupOperations()
        self.monitor_operations = MonitorManagementOperations()
        super().__init__()

    def get_tag_name_from_tags(self, tags: dict, tag_name: str):
        """
        This method returns the tag value by the tag_name
        :param tags:
        :type tags:
        :param tag_name:
        :type tag_name:
        :return:
        :rtype:
        """
        if tags:
            for key, value in tags.items():
                if Utils.equal_ignore_case(key, tag_name):
                    return value
        return ''

    def _delete_resource(self, resource_id: str):
        """
        This method deletes the
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        action = "deleted"
        try:
            if self._policy in ['instance_run', 'instance_idle']:
                action = "Stopped"
                delete_status = self.compute_operations.stop_vm(resource_id=resource_id)
            elif self._policy == 'unattached_volume':
                delete_status = self.compute_operations.delete_disk(resource_id=resource_id)
            elif self._policy == 'ip_unattached':
                delete_status = self.network_operations.release_public_ip(resource_id=resource_id)
            elif self._policy == 'unused_nat_gateway':
                delete_status = self.network_operations.delete_nat_gateway(resource_id=resource_id)
            logger.info(f'{self._policy} {action}: {resource_id}')
        except Exception as err:
            logger.info(f'Exception raised: {err}: {resource_id}')

    def update_resource_day_count_tag(self, resource_id: str, cleanup_days: int, tags: dict):
        tags = self._update_tag_value(tags=tags, tag_name='DaysCount', tag_value=str(cleanup_days))
        try:
            if self._policy in ['instance_run', 'unattached_volume', 'ip_unattached', 'unused_nat_gateway']:
                self.resource_group_operations.creates_or_updates_tags(resource_id=resource_id, tags=tags)
        except Exception as err:
            logger.info(f'Exception raised: {err}: {resource_id}')

    def _update_tag_value(self, tags: dict, tag_name: str, tag_value: str):
        """
        This method returns the updated tag_list by adding the tag_name and tag_value to the tags
        @param tags:
        @param tag_name:
        @param tag_value:
        @return:
        """
        if not tags:
            tags = {}
        if self._dry_run == "yes":
            tag_value = 0
        tag_value = f'{self.CURRENT_DATE}@{tag_value}'
        found = False
        updated_tags = {}
        if tags:
            for key, value in tags.items():
                if Utils.equal_ignore_case(key, tag_name):
                    if value.split("@")[0] != self.CURRENT_DATE:
                        updated_tags[key] = tag_value
                    else:
                        if int(tag_value.split("@")[-1]) == 0 or int(tag_value.split("@")[-1]) == 1:
                            updated_tags[key] = tag_value
                    found = True
        tags.update(updated_tags)
        if not found:
            tags.update({tag_name: tag_value})
        return tags

    def _get_all_instances(self):
        """
        This method returns the all instances list
        :return:
        :rtype:
        """
        return self.compute_operations.get_all_instances()

    def run_policy_operations(self):
        raise NotImplementedError("This method needs to be implemented")

    def _get_all_volumes(self) -> list:
        """
        This method returns the volumes by state
        :return:
        :rtype:
        """
        volumes = self.compute_operations.get_all_disks()
        return volumes

    def _get_active_cluster_ids(self):
        """
        This method returns the active cluster id's
        :return:
        :rtype:
        """
        active_instances = self._get_all_instances()
        cluster_ids = []
        for vm in active_instances:
            tags = vm.tags if vm.tags else {}
            for key, value in tags.items():
                if key.startswith('kubernetes.io/cluster'):
                    cluster_ids.append(key)
                    break
        return cluster_ids

    def _get_cluster_tag(self, tags: dict):
        """
        This method returns the cluster_tag
        :return:
        :rtype:
        """
        if tags:
            for key, value in tags.items():
                if key.startswith('kubernetes.io/cluster'):
                    return key
        return ''

    def _get_instance_status(self, resource_id: str, vm_name: str):
        """
        This method returns the VM status of the Virtual Machine
        :param resource_id:
        :type resource_id:
        :param vm_name:
        :type vm_name:
        :return:
        :rtype:
        """
        instance_statuses = self.compute_operations.get_instance_statuses(resource_id=resource_id, vm_name=vm_name)
        statuses = instance_statuses.get('statuses', {})
        if len(statuses) >= 2:
            status = statuses[1].get('display_status', '').lower()
        elif len(statuses) == 1:
            status = statuses[0].get('display_status', '').lower()
        else:
            status = 'Unknown Status'
        return status

    def __get_aggregation_metrics_value(self, metrics: dict, aggregation: str):
        """
        This method returns the aggregation value of the metrics
        :param metrics:
        :type metrics:
        :param aggregation:
        :type aggregation:
        :return:
        :rtype:
        """
        total_metrics = 0
        metric_aggregation_value = 0
        for metric in metrics.get('value', []):
            metrics_data = metric.get('timeseries', [])
            if metrics_data:
                total_metrics = len(metrics_data[0].get('data', []))
                for metric_data in metrics_data[0].get('data', []):
                    metric_aggregation_value += metric_data.get(aggregation)
        if Utils.equal_ignore_case(aggregation, 'average'):
            return round(metric_aggregation_value / total_metrics, DEFAULT_ROUND_DIGITS)
        else:
            return round(metric_aggregation_value, DEFAULT_ROUND_DIGITS)

    def get_cpu_utilization_percentage_metric(self, resource_id: str, days: int = INSTANCE_IDLE_DAYS):
        """
        This method returns the cpu utilization percentage
        :param days:
        :type days:
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        start_date, end_date = Utils.get_start_and_end_datetime(days=days)
        timespan = f'{start_date}/{end_date}'
        cpu_metrics = self.monitor_operations.get_resource_metrics(resource_id=resource_id,
                                                                   metricnames='Percentage CPU',
                                                                   aggregation='Average',
                                                                   timespan=timespan)
        average_cpu_metrics_value = self.__get_aggregation_metrics_value(metrics=cpu_metrics, aggregation='average')
        return average_cpu_metrics_value

    def get_network_in_kib_metric(self, resource_id: str, days: int = INSTANCE_IDLE_DAYS):
        """
        This method returns the total Network In KiB
        :param days:
        :type days:
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        start_date, end_date = Utils.get_start_and_end_datetime(days=days)
        timespan = f'{start_date}/{end_date}'
        network_in_metrics = self.monitor_operations.get_resource_metrics(resource_id=resource_id,
                                                                          metricnames='Network In Total',
                                                                          aggregation='Average',
                                                                          timespan=timespan)
        average_network_in_bytes = self.__get_aggregation_metrics_value(metrics=network_in_metrics,
                                                                        aggregation='average')
        return round(average_network_in_bytes / TOTAL_BYTES_IN_KIB, DEFAULT_ROUND_DIGITS)

    def get_network_out_kib_metric(self, resource_id: str, days: int = INSTANCE_IDLE_DAYS):
        """
        This method returns the total Network Out KiB
        :param days:
        :type days:
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        start_date, end_date = Utils.get_start_and_end_datetime(days=days)
        timespan = f'{start_date}/{end_date}'
        network_out_metrics = self.monitor_operations.get_resource_metrics(resource_id=resource_id,
                                                                           metricnames='Network Out Total',
                                                                           aggregation='Average',
                                                                           timespan=timespan)
        average_network_out_bytes = self.__get_aggregation_metrics_value(metrics=network_out_metrics,
                                                                         aggregation='average')
        return round(average_network_out_bytes / TOTAL_BYTES_IN_KIB, DEFAULT_ROUND_DIGITS)



from azure.mgmt.compute.v2023_03_01.models import VirtualMachine

from cloud_governance.cloud_resource_orchestration.clouds.azure.resource_groups.abstract_resource import \
    AbstractResource
from cloud_governance.cloud_resource_orchestration.utils.common_operations import check_name_and_get_key_from_tags
from cloud_governance.cloud_resource_orchestration.utils.constant_variables import DURATION, TICKET_ID


class MonitorCROResources(AbstractResource):
    """
    This class monitor CRO Resources
    """

    def __init__(self):
        super().__init__()

    def __get_common_data(self, tags: dict):
        """
        This method returns the common data
        :param tags:
        :type tags:
        :return:
        :rtype:
        """
        return {
            #  'instance_state': 'NA',  # virtual_machine.instance_view - not available
            'user_cro': self._compute_client.check_tag_name(tag_name='UserCRO', tags=tags),
            'user': self._compute_client.check_tag_name(tags=tags, tag_name='User'),
            'manager': self._compute_client.check_tag_name(tag_name='Manager', tags=tags),
            'approved_manager': self._compute_client.check_tag_name(tag_name='ApprovedManager', tags=tags),
            'owner': self._compute_client.check_tag_name(tag_name='Owner', tags=tags),
            'project': self._compute_client.check_tag_name(tag_name='Project', tags=tags),
            'email': self._compute_client.check_tag_name(tag_name='Email', tags=tags),
            'duration': self._compute_client.check_tag_name(tag_name='Duration', tags=tags, cast_type='int'),
            'estimated_cost': self._compute_client.check_tag_name(tag_name='EstimatedCost', tags=tags, cast_type='float')
        }

    def __monitor_instances(self):
        """
        This method monitors resources and returns the data
        :return:
        :rtype:
        """
        monitored_ticket_ids = {}
        cluster_tickets = {}
        virtual_machines: [VirtualMachine] = self._compute_client.get_all_instances()
        for virtual_machine in virtual_machines:
            name, tags = virtual_machine.name, virtual_machine.tags
            found_duration = self._compute_client.check_tag_name(tags=tags, tag_name=DURATION)
            if found_duration:
                ticket_id = self._compute_client.check_tag_name(tags=tags, tag_name=TICKET_ID)
                cluster_key, cluster_value = check_name_and_get_key_from_tags(tags=tags,
                                                                              tag_name='kubernetes.io/cluster/')
                hcp_key, hcp_name = check_name_and_get_key_from_tags(tags=tags, tag_name='api.openshift.com/name')
                if hcp_name:
                    cluster_key = hcp_name
                rosa, rosa_value = check_name_and_get_key_from_tags(tags=tags, tag_name='red-hat-clustertype')
                if cluster_key:
                    cluster_tickets.setdefault(ticket_id, {}).setdefault(cluster_key, {}).setdefault('instance_data',
                                                                                                     []) \
                        .append(f"{self._compute_client.check_tag_name(tag_name='Name', tags=tags)}: "
                                f"{virtual_machine.vm_id}: {virtual_machine.priority}: "
                                f"{virtual_machine.hardware_profile.vm_size}: {'rosa' if rosa else 'self'}")
                    cluster_tickets.setdefault(ticket_id, {}).setdefault(cluster_key, {}).update({
                        'region_name': virtual_machine.location,
                        'ticket_id': ticket_id,
                    })
                else:
                    resource_data = {
                        'region_name': virtual_machine.location,
                        'ticket_id': ticket_id,
                        'instance_data': f"{virtual_machine.name}: "
                                         f"{virtual_machine.vm_id}: {virtual_machine.priority}: "
                                         f"{virtual_machine.hardware_profile.vm_size}",
                    }
                    resource_data.update(self.__get_common_data(tags))
                    monitored_ticket_ids.setdefault(ticket_id, []).append(resource_data)

                for ticket_id, cluster_data in cluster_tickets.items():
                    for cluster_id, cluster_values in cluster_data.items():
                        cluster_values['cluster_name'] = cluster_id.split('/')[-1]
                        monitored_ticket_ids.setdefault(ticket_id, []).append(cluster_values)
                return monitored_ticket_ids
        return monitored_ticket_ids

    def run(self):
        """
        This method starts Azure CRO monitor
        :return:
        """
        return self.__monitor_instances()



import typeguard

from cloud_governance.common.logger.init_logger import logger


class DeleteIAMResources:
    """
    This class deleted the IAM resources
    IAM Role
    IAM User
    """

    def __init__(self, iam_client):
        """
        Initialize the aws clients
        :param iam_client:
        """
        self.iam_client = iam_client

    @typeguard.typechecked
    def delete_iam_zombie_resource(self, resource_id: str, resource_type: str):
        """
        This method checks for the which resource to delete
        :param resource_id:
        :param resource_type:
        :return:
        """
        if resource_type == 'iam_role':
            self.__delete_iam_role(resource_id)
        elif resource_type == 'iam_user':
            self.__delete_user(resource_id)

    @typeguard.typechecked
    def __delete_iam_role(self, resource_id: str):
        """
        This method deleted the zombie cluster iam role
        :param resource_id:
        :return:
        """
        try:
            # Detach policy from role
            role_policies = self.iam_client.list_attached_role_policies(RoleName=resource_id)
            if role_policies['AttachedPolicies']:
                for role_policy in role_policies['AttachedPolicies']:
                    self.iam_client.detach_role_policy(RoleName=resource_id, PolicyArn=role_policy.get('PolicyArn'))
                    self.iam_client.delete_policy(PolicyArn=role_policy.get('PolicyArn'))
            policy_names = self.iam_client.list_role_policies(RoleName=resource_id)
            if policy_names.get('PolicyNames'):
                for policy in policy_names.get('PolicyNames'):
                    self.iam_client.delete_role_policy(RoleName=resource_id, PolicyName=policy)
            instance_policies = self.iam_client.list_instance_profiles_for_role(RoleName=resource_id)
            if instance_policies['InstanceProfiles']:
                self.iam_client.remove_role_from_instance_profile(RoleName=resource_id,
                                                                  InstanceProfileName=resource_id.replace('role',
                                                                                                          'profile'))
            self.iam_client.delete_role(RoleName=resource_id)
            logger.info(f'delete_role: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot delete_role: {resource_id}, {err}')

    @typeguard.typechecked
    def __delete_user(self, resource_id: str):
        """
        This method deletes the Zombie cluster user
        :param resource_id:
        :return:
        """
        try:
            # Detach policy from user
            user_policies = self.iam_client.list_user_policies(UserName=resource_id)
            try:
                if user_policies['PolicyNames']:
                    self.iam_client.delete_user_policy(UserName=resource_id, PolicyName=f'{resource_id}-policy')
            except Exception:
                logger.exception(f'Cannot delete_policies: {user_policies.get("PolicyNames")}')
            list_access_key = self.iam_client.list_access_keys(UserName=resource_id)
            # @Todo user_delete permission is very problematic operation, it might affect other users.
            # delete user access key
            # for access_key in list_access_key['AccessKeyMetadata']:
            #     self.iam_client.delete_access_key(UserName=resource_id, AccessKeyId=access_key['AccessKeyId'])
            # self.iam_client.delete_user(UserName=resource_id)
            logger.info(f'delete_user: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot delete_user: {resource_id}, {err}')



from abc import ABC
from datetime import datetime
from typing import Union


from cloud_governance.common.elasticsearch.elasticsearch_operations import ElasticSearchOperations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.policy.policy_runners.common.abstract_upload import AbstractUpload


class UploadElasticSearch(AbstractUpload, ABC):

    DEFAULT_UPLOAD_LIMIT = 500

    def __init__(self):
        super().__init__()
        self._es_operations = ElasticSearchOperations()
        self.__es_host = self._environment_variables_dict.get('es_host', '')

    def upload(self, data: Union[list, dict]):
        """
        This method upload data to ElasticSearch
        :return:
        :rtype:
        """
        if self.__es_host:
            if self._es_operations.check_elastic_search_connection():
                if data:
                    if len(data) > self.DEFAULT_UPLOAD_LIMIT:
                        self._es_operations.upload_data_in_bulk(data_items=data.copy(), index=self._es_index)
                    else:
                        for policy_dict in data:
                            if 'RegionName' not in policy_dict:
                                policy_dict['RegionName'] = self._region
                            if 'account' not in policy_dict:
                                policy_dict['account'] = self._account
                            self._es_operations.upload_to_elasticsearch(data=policy_dict.copy(), index=self._es_index)
                    logger.info(f'Uploaded the policy results to elasticsearch index: {self._es_index}')
                else:
                    logger.error(f'No data to upload on @{self._account}  at {datetime.utcnow()}')
            else:
                logger.error('ElasticSearch host is not pingable, Please check your connection')
